package mi.hdm;

import javafx.application.Application;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.stage.Stage;
import mi.hdm.controllers.View;
import mi.hdm.filesystem.FileManager;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.InputStream;

public class GUIDriver extends Application {
    private final static Logger log = LogManager.getLogger(GUIDriver.class);

    private final static View STARTUP_VIEW = View.MAIN;
    private final static String PATH_TO_ICON = "/images/Tasty_Pages_Icon.png";
    private final static String FILE_NAME = "/music/GrandEtude_DagReinbott.wav";

    private static Stage stage;
    private static TastyPages model;
    private static GUIDriver app;

    private Thread musicThread;
    MusicPlayer runnable;

    public static void main(String[] args) throws InterruptedException {
        log.info("Starting TastyPages");
        try {
            model = FileManager.deserializeFromFilesystem();
        } catch (Exception e) {
            log.fatal("Internal exception while loading user data, check file paths and URIs");
            e.printStackTrace();
            System.exit(1);
        }

        /*musicThread = new Thread(() -> {
            final String ABSOLUTE_FILE_NAME = FileManager.getAbsolutePathFromResourceFolder(FILE_NAME);
            //!Thread.currentThread().isInterrupted()
            while(true) {
                playMusic(ABSOLUTE_FILE_NAME);
            }
        });*/

        launch(args);
        log.info("Shutting down application");
        log.info("Saving all data");
        FileManager.serializeToFile(model);
    }

    @Override
    public void start(Stage stage) throws Exception {
        app = this;
        GUIDriver.stage = stage;

        Parent parent = STARTUP_VIEW.getScene();
        final Scene scene = new Scene(parent, 1400, 800);

        stage.setTitle(STARTUP_VIEW.getWindowTitle());
        stage.setMaximized(true); //open scene in full screen
        stage.setScene(scene);
        stage.show();

        InputStream iconStream = getClass().getResourceAsStream(PATH_TO_ICON);
        if (iconStream != null) {
            stage.getIcons().add(new Image(iconStream));
            log.debug("Set icon for application");
        } else log.warn("Could not find icon for application. Check the filepath");

        runnable = new MusicPlayer(FILE_NAME);
        musicThread = new Thread(runnable);
        musicThread.setDaemon(true);
        log.info("Starting music thread.");
        musicThread.start();
    }

    @Override
    public void stop() {
        runnable.shutdown();
        log.debug("MusicPlayer was shut down");
    }

    public static TastyPages getModel() {
        return model;
    }

    public static GUIDriver getAppInstance() {
        return app;
    }

    public static Stage getApplicationStage() {
        return stage;
    }

    public static View getStartupView() {
        return STARTUP_VIEW;
    }

    public void pauseMusic() {
        log.debug("Pausing music");
        runnable.pause();
    }

    public void playMusic() {
        log.debug("Playing music");
        runnable.play();
    }

    /*public static void toggleCurrentMusicState() {
        musicIsOn = !musicIsOn;
        synchronized (musicThread) {
            if(musicIsOn) {
                log.debug("Notifying music thread");
                musicThread.notify();
            } else {
                try {
                    log.debug("Letting music thread wait...");
                    musicThread.wait();
                } catch (InterruptedException e) {
                    log.error("Could not pause music: {}", e.getMessage());
                    e.printStackTrace();
                }
            }
        }
    }*/
}
