package mi.hdm.components;

import javafx.scene.Cursor;
import javafx.scene.control.Label;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.VBox;
import mi.hdm.recipes.Category;
import mi.hdm.recipes.CategoryManager;
import mi.hdm.recipes.Recipe;

import java.util.ArrayList;
import java.util.List;

/**
 * Box to display a recipe preview in a grid layout
 */
public class RecipeVbox extends VBox {
    private final Recipe recipe;

    public RecipeVbox(Recipe recipe, CategoryManager categoryManager) {
        super();
        this.recipe = recipe;

        Label recipeName = new Label(recipe.getName());
        recipeName.setStyle(
                "-fx-font-weight: 800;" +
                        "-fx-font-scale: 16"
        );
        String desc = recipe.getDescription();
        Label recipeDescription = new Label(desc.substring(0, Math.min(desc.length(), 80))); //maximum of 80 characters for the recipe description
        List<CategoryPreviewLabel> categoryLabels = new ArrayList<>();

        //add a maximum of 3 categories to the recipe tile
        List<Category> categories = categoryManager.getCategoriesFromKeys(recipe.getCategoryCodes());
        for (int i = 0; i < Math.min(recipe.getCategoryCodes().size(), 3); i++) {
            CategoryPreviewLabel l = new CategoryPreviewLabel(categories.get(i));
            categoryLabels.add(l);
        }
        FlowPane categoryFlowPane = new FlowPane();
        categoryFlowPane.setHgap(5.0);
        categoryFlowPane.getChildren().addAll(categoryLabels);

        ImageView recipeImage = new ImageView(new Image(recipe.getImageURL().toString()));
        recipeImage.setPreserveRatio(true);
        recipeImage.setFitWidth(150);
        recipeImage.setFitHeight(150);

        this.getChildren().addAll(recipeImage, recipeName, recipeDescription, categoryFlowPane);

        this.setPrefWidth(180);
        this.setPrefHeight(250);
        this.setStyle(
                "-fx-background-color: #DEDEDE;" +
                        "-fx-border-color: #D91C1C;" +
                        "-fx-border-radius: 8;" +
                        "-fx-border-width: 2;" +
                        "-fx-padding: 8;"
        );
        this.setCursor(Cursor.HAND);
    }

    public Recipe getAssociatedRecipe() {
        return recipe;
    }
}
