package mi.hdm.controllers;

import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Label;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.Pane;
import javafx.stage.Stage;
import mi.hdm.GUIDriver;
import mi.hdm.recipes.NutritionTable;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;

public abstract class BaseController {
    private final static Logger log = LogManager.getLogger(BaseController.class);

    protected static Scene currentScene;
    private static View currentView = GUIDriver.getStartupView();

    protected void changeScene(final View newScene, Object... args) {
        if (newScene == currentView) {
            return;
        }
        final Stage stage = GUIDriver.getApplicationStage();
        try {
            //get current state of window
            boolean wasMaximized = stage.isMaximized();
            double sizeX = stage.getWidth();
            double sizeY = stage.getHeight();
            //get title for new scene
            String newWindowTitle = newScene.getWindowTitle();

            log.debug("Attempting to change the scene to {}", newScene);
            Parent newParent = newScene.getScene(args);

            currentScene = new Scene(newParent, sizeX, sizeY);
            currentView = newScene;

            stage.setTitle(newWindowTitle);
            stage.setScene(currentScene);
            stage.setMaximized(wasMaximized);
            log.info("new view: {}", currentView);
        } catch (IOException io) {
            log.error("IO Exception: something went wrong when changing the scene.");
            io.printStackTrace();
        }
    }

    protected void loadHeader(AnchorPane parent) {
        try {
            //load the header and constrain it to the left and top borders of the parent element
            AnchorPane header = (AnchorPane) ViewComponent.HEADER.getParentElement();
            AnchorPane.setLeftAnchor(header, 0.0);
            AnchorPane.setTopAnchor(header, 0.0);

            parent.getChildren().add(header);
        } catch (IOException e) {
            log.error("Something went wrong loading the header");
            e.printStackTrace();
        }
    }

    /**
     * Wrapper function that loads the NutritionTable component and appends it to the specified target pane
     *
     * @param target         Nutrition table will be appended to this target
     * @param nutritionTable Nutrition table that will be drawn
     * @param text           Descriptive text for UI
     * @param days           How many days this nutrition table spans across
     * @see ViewComponent
     */
    protected void putNutritionTable(Pane target, NutritionTable nutritionTable, String text, int days) {
        target.getChildren().clear();
        try {
            target.getChildren().add(ViewComponent.NUTRITION_TABLE.getParentElement(nutritionTable, text, days));
        } catch (IOException e) {
            log.error("Could not load nutrition table. Displaying placeholder");
            e.printStackTrace();
            target.getChildren().add(new Label("Placeholder - nutrition table couldn't be loaded!"));
        }
    }
}
