package mi.hdm.controllers;

import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.ColorPicker;
import javafx.scene.control.TextField;
import javafx.scene.layout.AnchorPane;
import javafx.scene.paint.Color;
import mi.hdm.exceptions.InvalidCategoryException;
import mi.hdm.recipes.CategoryManager;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class CategoryCreatorController extends BaseController {
    private final static Logger log = LogManager.getLogger(CategoryCreatorController.class);

    private final CategoryManager categoryManager;

    @FXML
    private AnchorPane parent;
    @FXML
    private TextField nameTextField;
    @FXML
    private ColorPicker colorPicker;

    public CategoryCreatorController(CategoryManager categoryManager) {
        this.categoryManager = categoryManager;
    }

    @FXML
    public void initialize() {
        loadHeader(parent);
    }

    @FXML
    public void handleAddCategory() {
        log.debug("User submitted category");
        String name = nameTextField.getText();
        int colorCode = colorToHexInt(colorPicker.getValue());
        try {
            categoryManager.addCategory(name, colorCode);
            log.info("Created new category '{}'.", name);
            changeScene(View.MAIN);
        } catch (InvalidCategoryException e) {
            Alert a = new Alert(Alert.AlertType.ERROR);
            a.setHeaderText("Error creating category");
            a.setContentText(e.getMessage());
            a.show();
            log.error("Category '{}' was not created;", name);
            log.error(e.getMessage());
        }
    }

    @FXML
    public void handleCancel() {
        log.info("User canceled category creation");
        changeScene(View.MAIN);
    }

    private String format(double val) {
        String in = Integer.toHexString((int) Math.round(val * 255));
        return in.length() == 1 ? "0" + in : in;
    }

    private int colorToHexInt(Color value) {
        return Integer.parseInt(format(value.getRed()) + format(value.getGreen()) + format(value.getBlue()), 16);
    }
}
