package mi.hdm.controllers.components;

import javafx.fxml.FXML;
import javafx.scene.control.ProgressBar;
import javafx.scene.paint.Color;
import javafx.scene.text.Text;
import mi.hdm.recipes.Nutrition;
import mi.hdm.recipes.NutritionTable;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.math.BigDecimal;
import java.util.Map;

public class NutritionTableController {
    private final static Logger log = LogManager.getLogger(NutritionTableController.class);

    private final NutritionTable nutritionTable;
    private final String text;
    private final int days;

    @FXML
    private ProgressBar caloriesProgressBar;
    @FXML
    private ProgressBar carbsProgressBar;
    @FXML
    private ProgressBar fatProgressbar;
    @FXML
    private ProgressBar proteinProgressBar;
    @FXML
    private ProgressBar fibersProgressBar;
    @FXML
    private ProgressBar saltProgressBar;
    @FXML
    private Text caloriesPercentage;
    @FXML
    private Text carbsPercentage;
    @FXML
    private Text fatPercentage;
    @FXML
    private Text proteinPercentage;
    @FXML
    private Text fiberPercentage;
    @FXML
    private Text saltPercentage;
    @FXML
    private Text description;

    public NutritionTableController(NutritionTable nutritionTable, String text, int days) {
        this.nutritionTable = nutritionTable;
        this.text = text;
        this.days = days;
    }

    @FXML
    public void initialize() {
        description.setText(text);
        drawNutritionTable();
    }

    private void drawNutritionTable() {
        log.debug("Drawing nutrition table for meal plan.");
        final Map<Nutrition, BigDecimal> weeklyNutrition = nutritionTable.getTable();
        //calculate percentage of weekly recommended dosage
        double cals = weeklyNutrition.get(Nutrition.CALORIES).doubleValue() / (NutritionTable.CALORIES_DAILY * days);
        double carbs = weeklyNutrition.get(Nutrition.CARBS).doubleValue() / (NutritionTable.CARBS_DAILY * days);
        double fat = weeklyNutrition.get(Nutrition.FAT).doubleValue() / (NutritionTable.FAT_DAILY * days);
        double protein = weeklyNutrition.get(Nutrition.PROTEINS).doubleValue() / (NutritionTable.DAILY_PROTEIN * days);
        double fiber = weeklyNutrition.get(Nutrition.FIBERS).doubleValue() / (NutritionTable.DAILY_FIBER * days);
        double salt = weeklyNutrition.get(Nutrition.SALT).doubleValue() / (NutritionTable.DAILY_SALT * days);
        //update gui
        setPercentage(caloriesProgressBar, caloriesPercentage, cals);
        setPercentage(carbsProgressBar, carbsPercentage, carbs);
        setPercentage(fatProgressbar, fatPercentage, fat);
        setPercentage(proteinProgressBar, proteinPercentage, protein);
        setPercentage(fibersProgressBar, fiberPercentage, fiber);
        setPercentage(saltProgressBar, saltPercentage, salt);
    }

    private void setPercentage(ProgressBar progressBar, Text textObject, double value) {
        log.debug("Setting percentage for {}", textObject.getId());
        progressBar.setProgress(value);
        textObject.setText(Math.round(value * 100) + "%");
        //set text color for percentages
        if (value > 0.9) {
            if (value > 1) {
                textObject.setFill(Color.valueOf("#D91C1C")); //red
            }
            textObject.setFill(Color.valueOf("#ff8a0d")); //orange
        } else {
            textObject.setFill(Color.valueOf("#000000")); //default: black
        }
    }
}
